@@ -1,15 +1,17 @@
 from logging import getLogger
-from typing import Any, Callable, List
+from typing import Any, Callable, List, Optional, Tuple
 
 from sqlalchemy import Table, text
 
-from watchmen_model.admin import Topic
+from watchmen_model.admin import FactorType, Topic
 from watchmen_storage import as_table_name, EntityCriteria, EntitySort, Literal
-from watchmen_storage_rds import build_sort_for_statement, SQLAlchemyStatement, StorageRDS, TopicDataStorageRDS, build_offset_for_statement
+from watchmen_storage_rds import build_offset_for_statement, build_sort_for_statement, SQLAlchemyStatement, \
+	StorageRDS, TopicDataStorageRDS
 from .table_creator import build_aggregate_assist_column, build_columns, build_columns_script, build_indexes_script, \
 	build_unique_indexes_script, build_version_column
 from .where_build import build_criteria_for_statement, build_literal
 
+# noinspection DuplicatedCode
 logger = getLogger(__name__)
 
 
@@ -26,11 +28,14 @@ class StorageOracle(StorageRDS):
 	def build_offset_for_statement(
 			self, statement: SQLAlchemyStatement, page_size: int, page_number: int) -> SQLAlchemyStatement:
 		offset = page_size * (page_number - 1)
-		return statement.offset(offset).fetch(page_size)
+		stmt = text(str(
+			statement.compile(
+				compile_kwargs={"literal_binds": True})) + f" OFFSET {offset} ROWS FETCH NEXT {page_size} ROWS ONLY")
+		return stmt
 
 
 class TopicDataStorageOracle(StorageOracle, TopicDataStorageRDS):
-	# noinspection SqlResolve
+	# noinspection SqlResolve,DuplicatedCode
 	def create_topic_entity(self, topic: Topic) -> None:
 		try:
 			self.connect()
@@ -64,6 +69,7 @@ CREATE TABLE {entity_name} (
 		finally:
 			self.close()
 
+	# noinspection DuplicatedCode
 	def update_topic_entity(self, topic: Topic, original_topic: Topic) -> None:
 		"""
 		1. drop no column,\n
@@ -116,5 +122,52 @@ CREATE TABLE {entity_name} (
 		finally:
 			self.close()
 
+	# noinspection SqlResolve
+	def ask_synonym_columns_sql(self, table_name: str) -> str:
+		return \
+			f"SELECT UTC.TABLE_NAME, UTC.COLUMN_NAME, UTC.DATA_TYPE, " \
+			f"CASE " \
+			f"WHEN DATA_PRECISION IS NOT NULL THEN DATA_TYPE || '(' || DATA_PRECISION || ',' || DATA_SCALE || ')' " \
+			f"WHEN DATA_LENGTH IS NOT NULL THEN DATA_TYPE || '(' || DATA_LENGTH || ')' " \
+			f"ELSE DATA_TYPE " \
+			f"END AS COLUMN_TYPE, " \
+			f"UCC.COMMENTS AS COLUMN_COMMENT " \
+			f"FROM USER_TAB_COLUMNS UTC, USER_COL_COMMENTS UCC " \
+			f"WHERE UTC.TABLE_NAME = UPPER('{table_name}') AND UCC.TABLE_NAME = UPPER('{table_name}') " \
+			f"AND UTC.COLUMN_NAME = UCC.COLUMN_NAME " \
+			f"ORDER BY UTC.COLUMN_ID"
+
+	def schema_column_data_type_to_factor_type(self, schema_column_data_type: str) -> Tuple[FactorType, Optional[str]]:
+		if '(' in schema_column_data_type:
+			index = schema_column_data_type.index('(')
+			precision = schema_column_data_type[index + 1: len(schema_column_data_type) - 1]
+			data_type = schema_column_data_type[: index].upper()
+		else:
+			precision = None
+			data_type = schema_column_data_type.upper()
+
+		if data_type in ['NUMBER', 'FLOAT', 'LONG', 'BINARY_FLOAT', 'BINARY_DOUBLE']:
+			return FactorType.NUMBER, precision
+		elif data_type in ['DATE', 'TIMESTAMP']:
+			return FactorType.DATETIME, None
+		elif data_type in [
+			'VARCHAR2', 'NVARCHAR2', 'CHAR', 'NCHAR'
+		]:
+			return FactorType.TEXT, precision
+		elif data_type in ['CLOB', 'NCLOB', 'BLOB']:
+			return FactorType.OBJECT, None
+		else:
+			return FactorType.TEXT, None
+
+	# noinspection SqlResolve,SqlCaseVsIf
+	def ask_synonym_indexes_sql(self, table_name: str) -> str:
+		return \
+			f"SELECT UI.TABLE_NAME, UIC.COLUMN_NAME, UI.INDEX_NAME, " \
+			f"CASE WHEN UI.UNIQUENESS = 'UNIQUE' THEN 0 ELSE 1 END AS NON_UNIQUE " \
+			f"FROM USER_IND_COLUMNS UIC, USER_INDEXES UI " \
+			f"WHERE UIC.TABLE_NAME = UPPER('{table_name}') AND UI.TABLE_NAME = UPPER('{table_name}') " \
+			f"AND UIC.INDEX_NAME = UI.INDEX_NAME " \
+			f"ORDER BY NON_UNIQUE, UI.INDEX_NAME, UIC.COLUMN_NAME"
+
 	def build_literal(self, tables: List[Table], a_literal: Literal, build_plain_value: Callable[[Any], Any] = None):
 		return build_literal(tables, a_literal, build_plain_value)
